//    Uploaded.to API - Java
//    Copyright (C) 2012  Julius Fischer webmaster@it-gecko.de
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package to.uploaded;

import java.io.File;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.http.NameValuePair;
import org.apache.http.ParseException;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.message.BasicNameValuePair;
import org.json.simple.JSONObject;
import org.json.simple.JSONValue;

import to.uploaded.account.UPAccount;
import to.uploaded.exception.LoginFailedException;
import to.uploaded.file.UPDownload;
import to.uploaded.file.UPFile;
import to.uploaded.file.UPFileHandler;
import to.uploaded.file.UPFileList;
import to.uploaded.file.UPUpload;
import to.uploaded.folder.UPFolderHandler;
import to.uploaded.http.UPHttp;

/**
* Uploaded.to API - JAVA Class
* 
*  Untersttzung von:
*  - hochladen, herunterladen, Backup, lschen und umbenennen von Dateien 
*  - erstellen, lschen und umbenennen von Ordnern 
*  - Zuordnung von Dateien in Ordnern. 
*  - Remote Uploads
*  - Free und Premium User Support
*   
* @author Julius Fischer
* @copyright 2011 Julius Fischer
* 
* @license GNU LGPL <br>
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
* 
* @link http:\\www.it-gecko.de
* @version 0.5
*
*/
public class Uploaded {
	
	private UPHttp upHttp = new UPHttp();
	private UPFileHandler upFileHandler = null;
	private UPFolderHandler upFolderHandler = null;
	
	private String userid, password, authStr;

	/**
	 * Constructor ohne Login
	 */
	public Uploaded() {
		
	}
	
	/**
	 * Constructor mit Login
	 * 
	 * @param userid User-Id
	 * @param password User-Password
	 * @throws LoginFailedException
	 */
	public Uploaded(String userid, String password) throws LoginFailedException
	{
		this.userid = userid;
		this.password = password;
		
		initLogin();
		
		try {
			
			String page = this.upHttp.get("http://uploaded.to/");
			Matcher mat = Pattern.compile("\\<input type\\=\"hidden\" id\\=\"user_pw\" value\\=\"([a-z0-9]+)\" /\\>").matcher(page);
			
			if(!mat.find())
				throw new LoginFailedException();
			
			this.authStr  = "&id=" + userid + "&pw=" + mat.group(1);
			
		} catch (ClientProtocolException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}		
	}
	
	/**
	 * Gibt Account Infos 
	 * @return
	 * @throws LoginFailedException
	 */
	public UPAccount getAccountInfos() throws LoginFailedException
	{		
		List<NameValuePair> formparams = new ArrayList<NameValuePair>();
		
		formparams.add(new BasicNameValuePair("uid", userid));
		formparams.add(new BasicNameValuePair("upw", password));
		
		String content = "";
		
		try {
			content = this.upHttp.post("http://uploaded.to/status", formparams, false, true);
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
				
		Matcher mat = Pattern.compile("blocked|wrong user|wrong password").matcher(content);
		
		if(mat.find())
			throw new LoginFailedException();
		
		UPAccount upAccount = new UPAccount();
		
		mat = Pattern.compile("status: (\\w+)").matcher(content);
		
		if(mat.find())
			upAccount.setStatus(mat.group(1));
		
		mat = Pattern.compile("traffic: (\\d+)").matcher(content);
		
		if(mat.find())
			upAccount.setTraffic(Long.parseLong(mat.group(1)));
		
		mat = Pattern.compile("expire: (\\d+)").matcher(content);
		
		if(mat.find())
			upAccount.setExpire(Long.parseLong(mat.group(1)));
		
		return upAccount;
	}
	
	public UPHttp getUpHttp()
	{
		return upHttp;
	}
		
	
	/**
	 * @return the authStr
	 */
	public String getAuthStr() {
		return authStr;
	}

	private void initLogin() throws LoginFailedException
	{
		
		List<NameValuePair> formparams = new ArrayList<NameValuePair>();
		
		formparams.add(new BasicNameValuePair("id", userid));
		formparams.add(new BasicNameValuePair("pw", password));
		
		String content = "";
		
		try {
			content = this.upHttp.post("http://uploaded.to/io/login", formparams, false, true);
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		if(!content.equals(""))
		{
			throw new LoginFailedException();
		}
	}
	
	/**
	 * Startet einen Remote-Upload
	 * 
	 * @param url Remote URL
	 * @return true = Ok | false = Failed
	 */
	public boolean addRemoteUpload(String url)
	{		
		try {
			
			List<NameValuePair> formparams = new ArrayList<NameValuePair>();
			
			formparams.add(new BasicNameValuePair("values", URLEncoder.encode(url, "UTF-8")));
			
			return upHttp.post("http://uploaded.to/io/remote/add", formparams).equals("<em class=\'cG\'>Added</em>: " + url + "\n");
			
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}
	
	/**
	 * Importiert eine URL
	 * 
	 * @param upLink URL
	 * @return UPFile der neuen Datei
	 */
	public UPFile importFile(String upLink)
	{		
		String content = "";
		
		try {
			
			List<NameValuePair> formparams = new ArrayList<NameValuePair>();
			
			formparams.add(new BasicNameValuePair("urls", upLink));
			
			content = upHttp.post("http://uploaded.to/io/import", formparams);
			
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		/*
		 * Uploaded.to ...
		 */
		content = content.substring(0, content.length()-1);
		
		Matcher mat = Pattern.compile("([{,])(\\s*)([^\"]+?)\\s*:").matcher(content);
		
		while(mat.find())
		{
			content = content.replaceAll(mat.group(3), "\"" + mat.group(3) + "\"");
		}
		
		
		JSONObject jsonObject = (JSONObject)JSONValue.parse(content);

		if(jsonObject == null || jsonObject.get("err") != null)
			return null;
		
		return upFileHandler.fetchFileList().findId((String) jsonObject.get("newAuth"));
	}
	
	/**
	 * Sicherheitskopie einer Datei anlegen
	 * @param fileId File-ID der zu kopierenden Datei
	 * @return UPFile der neuen Datei
	 */
	public UPFile backupFile(String fileId)
	{
		String content = "";
		
		try {
			content = upHttp.get("http://uploaded.to/io/file/backup/" + fileId);
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		if(content.isEmpty())
			return null;
		
		/*
		 * Uploaded.to ...
		 */
		content = content.replace("auth", "\"auth\"");
		content = content.replace("filename", "\"filename\"");
		content = content.replace("size", "\"size\"");
		
		JSONObject jsonObject = (JSONObject)JSONValue.parse(content);
				
		if(jsonObject == null || jsonObject.get("err") != null)
			return null;
				
		UPFileList upFileList = upFileHandler.fetchFileList();
				
		return upFileList.findId((String) jsonObject.get("auth"));
	}
	
	/**
	 *  
	 * @param file Datei die hochgeladen werden soll
	 * @return UPUpload
	 */
	public UPUpload factoryUpload(File file)
	{
		return new UPUpload(this, file);
	}
	
	/**
	 * 
	 * @param path Speicherort fr den Download
	 * @param fileId File-ID
	 * @return UPDownload
	 */
	public UPDownload factoryDownload(String path, String fileId)
	{
		return new UPDownload(this, path, fileId);
	}
	
	/**
	 * 
	 * @return
	 */
	public UPFileHandler getUPFileHandler()
	{
		if(upFileHandler == null)
			upFileHandler = new UPFileHandler(upHttp);
		
		return upFileHandler;
	}
	
	/**
	 * 
	 * @return
	 */
	public UPFolderHandler getUPFolderHandler()
	{
		if(upFolderHandler == null)
			upFolderHandler = new UPFolderHandler(upHttp);
		
		return upFolderHandler;
	}

}
