//    Uploaded.to API - Java
//    Copyright (C) 2012  Julius Fischer webmaster@it-gecko.de
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package to.uploaded.file;

import java.io.File;
import java.io.IOException;
import java.util.Random;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpVersion;
import org.apache.http.ParseException;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.mime.content.FileBody;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.CoreProtocolPNames;
import org.apache.http.util.EntityUtils;

import to.uploaded.Uploaded;
import to.uploaded.exception.UploadFailedExecption;

public class UPUpload extends UPEventListener{
	
	private File file;
	
	private long progressByte = 0;
	private long contentLength = 0;
	private String fileId = null;
	private Uploaded uploaded = null;
	
	public UPUpload(Uploaded uploaded, File file)
	{
		this.uploaded = uploaded;
		this.file = file;
	}
	
	/**
	 * Gibt File-ID zurck wenn der Uploadvorgang erfolgreich abgeschlossen wurde
	 * @return
	 */
	public String getFileId()
	{
		return fileId;
	}
	
	/**
	 * Gibt die totale Uploadgre zurck
	 * @return
	 */
	public long getLength()
	{
		return contentLength;
	}
	
	/**
	 * Upload abgeschlossen
	 * @return true = fertig
	 */
	public boolean isFinish()
	{
		return contentLength == progressByte;
	}
	
	/**
	 * Startet den Uploadvorgang
	 * @throws UploadFailedExecption
	 */
	public void start() throws UploadFailedExecption
	{		
		DefaultHttpClient httpclient 	= new DefaultHttpClient();
		HttpPost httppost 				= new HttpPost(getUploadServer() + "upload?admincode=" + UPUpload.getEditKey() + uploaded.getAuthStr());
		UPMultipartEntity mpEntity 		= new UPMultipartEntity(this);
		
		httpclient.getParams().setParameter(CoreProtocolPNames.PROTOCOL_VERSION, HttpVersion.HTTP_1_1);	
					
		mpEntity.addPart("Filedata", new FileBody(file));
		
		contentLength = mpEntity.getContentLength();

		httppost.setEntity(mpEntity);

		HttpResponse response = null;
		
		try {
			
			response = httpclient.execute(httppost);
			
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		HttpEntity resEntity = response.getEntity();
		String content = "";
		
		if (resEntity != null) {
			try {
				content = EntityUtils.toString(resEntity);
			} catch (ParseException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		httpclient.getConnectionManager().shutdown();
		
		if(content.isEmpty())
		{
			fileId = null;
			throw new UploadFailedExecption("Upload fehlgeschlagen");
		}
		
		fileId = content.substring(0, content.indexOf(","));
		
		httpclient.getConnectionManager().shutdown();
	}

	public String getUploadServer(){

		String content = "";

		try {
			content = uploaded.getUpHttp().get("http://uploaded.to/api/uploadserver");
		} catch (ClientProtocolException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		return content;
	}

	public void updateProgressByte(long i)
	{
		progressByte = progressByte + i;
		
		notifyListeners(this);
	}
	
	/**
	 * Gibt die bereits verarbeitete bytelnge zurck
	 * @return
	 */
	public long getProgressByte()
	{
		return progressByte;
	}

	public static String getEditKey()
	{		
		char[] con = {'b','c','d','f','g','h','j','k','l','m','n','p','r','s','t','v','w','x','y','z'};
		char[] voc = {'a','e','i','o','u'};
		int len = 3, a, b;
		String pw = "";
		Random rnd = new Random();
		
		for(int i = 0; i < len; i++)
		{
			a = Math.abs(rnd.nextInt()) % 20;
			b = Math.abs(rnd.nextInt()) % 5;
			pw = pw + con[a] + voc[b];
		}
		
		return pw;
	}
	
	public static interface UPUploadProgress
	{
		void progress(UPUpload upUpload);
	}
}
