//    Uploaded.to API - Java
//    Copyright (C) 2012  Julius Fischer webmaster@it-gecko.de
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

package to.uploaded.http;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.List;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.ParseException;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.CookieStore;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;

public class UPHttp {
	
	private CookieStore cookieStore = null;
	
	public UPHttp()
	{
		
	}
	
	public String get(String url) throws ClientProtocolException, IOException
	{
		DefaultHttpClient httpclient 	= new DefaultHttpClient();
		
		if(cookieStore != null)
			httpclient.setCookieStore(cookieStore);
		
		HttpResponse response 			= httpclient.execute(new HttpGet(url));
		HttpEntity entity 				= response.getEntity();
		
		String content =  EntityUtils.toString(entity);
		httpclient.getConnectionManager().shutdown();
		
		return content;
		
	}
	
	public String post(String url) throws ParseException, ClientProtocolException, IOException
	{
		return post(url, null, true, false);
	}
	
	public String post(String url, List<NameValuePair> formparams) throws ParseException, ClientProtocolException, IOException
	{
		return post(url, formparams, true, false);
	}
	
	public String post(String url, List<NameValuePair> formparams, boolean useCookieStore, boolean saveCookie) throws ParseException, ClientProtocolException, IOException
	{	
		HttpPost httppost				= new HttpPost(url);
		DefaultHttpClient httpclient	= new DefaultHttpClient();
		
		httppost.addHeader("Referer", "http://uploaded.to/upload");
		
		if(formparams != null)
		{
			try {
				httppost.setEntity(new UrlEncodedFormEntity(formparams, "UTF-8"));
			} catch (UnsupportedEncodingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	
		if(useCookieStore && cookieStore != null)
			httpclient.setCookieStore(this.cookieStore);
		
		if(saveCookie)	
			this.cookieStore = httpclient.getCookieStore();

		String content =  EntityUtils.toString(httpclient.execute(httppost).getEntity());
		
		httpclient.getConnectionManager().shutdown();
		
		return content;
	}

	/**
	 * @return the cookieStore
	 */
	public CookieStore getCookieStore() {
		return cookieStore;
	}

	/**
	 * @param cookieStore the cookieStore to set
	 */
	public void setCookieStore(CookieStore cookieStore) {
		this.cookieStore = cookieStore;
	}

}
